#include "queue.h"

CQueue::CQueue() noexcept : queue(nullptr), size(0), capacity(0) {}

CQueue::CQueue(int initialCapacity) : queue(new double[initialCapacity]), size(0), capacity(initialCapacity) {}

CQueue::CQueue(std::istream& is) : queue(nullptr), size(0), capacity(0) {
    std::cout << "Enter the number of elements: ";
    int count;
    if (!(is >> count)) {
        throw std::ios_base::failure("Input error while reading number of elements");
    }

    for (int i = 0; i < count; ++i) {
        double value;
        std::cout << "Enter element " << i + 1 << ": ";
        if (!(is >> value)) {
            throw std::ios_base::failure("Input error while reading elements");
        }
        addtoqueue(value);
    }
}

CQueue::CQueue(const CQueue& other) : size(other.size), capacity(other.capacity) {
    queue = std::make_unique<double[]>(capacity);
    for (int i = 0; i < size; ++i) {
        queue[i] = other.queue[i];
    }
}

CQueue::~CQueue() = default;

void CQueue::addtoqueue(double value) {
    if (size == capacity) {
        int newCapacity = (capacity == 0) ? 1 : capacity * 2;
        std::unique_ptr<double[]> newQueue(new double[newCapacity]);
        for (int i = 0; i < size; ++i) {
            newQueue[i] = queue[i];
        }
        queue = std::move(newQueue);
        capacity = newCapacity;
    }
    queue[size++] = value;
}

void CQueue::deletefromqueue() {
    if (size > 0) {
        for (int i = 1; i < size; ++i) {
            queue[i - 1] = queue[i];
        }
        size--;
    }
}

int CQueue::getSize() const noexcept {
    return size;
}

double CQueue::getMax() const {
    if (size == 0) {
        throw std::runtime_error("Queue is empty");
    }
    double max = queue[0];
    for (int i = 1; i < size; ++i) {
        if (queue[i] > max) {
            max = queue[i];
        }
    }
    return max;
}

double CQueue::getMin() const {
    if (size == 0) {
        throw std::runtime_error("Queue is empty");
    }
    double min = queue[0];
    for (int i = 1; i < size; ++i) {
        if (queue[i] < min) {
            min = queue[i];
        }
    }
    return min;
}

void CQueue::clear() noexcept {
    size = 0;
    capacity = 0;
    queue.reset();
}

bool CQueue::isEmpty() const noexcept {
    return size == 0;
}

double CQueue::getQueueElement(int index) const {
    if (index < 0 || index >= size) {
        throw std::out_of_range("Index out of range");
    }
    return queue[index];
}

CQueue CQueue::operator+(const CQueue& other) const {
    if (size != other.size) {
        throw std::logic_error("Queues are of different sizes");
    }

    CQueue result(size);

    for (int i = 0; i < size; ++i) {
        result.addtoqueue(queue[i] + other.queue[i]);
    }

    return result;
}

CQueue CQueue::operator-(const CQueue& other) const {
    if (size != other.size) {
        throw std::logic_error("Queues are of different sizes");
    }

    CQueue result(size);

    for (int i = 0; i < size; ++i) {
        result.addtoqueue(queue[i] - other.queue[i]);
    }

    return result;
}

CQueue CQueue::operator*(const double scalar) const {
    CQueue result(size);

    for (int i = 0; i < size; ++i) {
        result.addtoqueue(queue[i] * scalar);
    }

    return result;
}

std::ostream& operator<<(std::ostream& os, const CQueue& queue) {
    if (!os) {
        throw std::ios_base::failure("Output stream error");
    }
    for (int i = 0; i < queue.size; ++i) {
        os << queue.queue[i] << " ";
    }
    return os;
}

std::istream& operator>>(std::istream& is, CQueue& queue) {
    if (!is) {
        throw std::ios_base::failure("Input stream error");
    }
    double value;
    is >> value;
    queue.addtoqueue(value);
    return is;
}

